

#ifndef  __INSTRUCTION_H__
#define  __INSTRUCTION_H__


#include <stdint.h>
#include <vector>


extern "C++" {
	namespace v1processor {
		using namespace std;

		class Instruction;

		class InstructionFactory {
			public:
				Instruction *newInstruction(vector<string> &labels, string id);
		};

		class Instruction {
			public:
				vector<string> labels;
				string mnemonic;
				uint16_t getRegister(string &registerName);
				Instruction(vector<string> &labels, string mnemonic) : labels(labels), mnemonic(mnemonic) { };
				virtual uint16_t getBinary() = 0;
		};

		class LoadIInstruction : public Instruction {
			public:
				string sourceLabel;
				int16_t sourceValue;
				LoadIInstruction(vector<string> &labels) : Instruction(labels, "loadi") { };
				uint16_t getBinary();
		};

		class LoadInstruction : public Instruction {
			public:
				LoadInstruction(vector<string> &labels) : Instruction(labels, "load") { };
				uint16_t getBinary() { return 0x1000; };
		};

		class OpInstruction : public Instruction {
			public:
				string destinationRegister;
				string sourceRegister;
				string operation;
				OpInstruction(vector<string> &labels) : Instruction(labels, "op") { };
				uint16_t getBinary();
		};

		class StoreInstruction : public Instruction {
			public:
				StoreInstruction(vector<string> &labels) : Instruction(labels, "store") { };
				uint16_t getBinary() { return 0x3000; };
		};

		class PushInstruction : public Instruction {
			public:
				PushInstruction(vector<string> &labels) : Instruction(labels, "push") { };
				uint16_t getBinary() { return 0x4000; };
		};

		class PopInstruction : public Instruction {
			public:
				PopInstruction(vector<string> &labels) : Instruction(labels, "pop") { };
				uint16_t getBinary() { return 0x5000; };
		};

		class AbstractJInstruction : public Instruction {
			public:
				string destinationLabel;
				int16_t destinationValue;
				AbstractJInstruction(vector<string> &labels, string mnemonic) : Instruction(labels, mnemonic) { };
		};

		class JInstruction : public AbstractJInstruction {
			public:
				JInstruction(vector<string> &labels) : AbstractJInstruction(labels, "j") { };
				uint16_t getBinary();
		};

		class JzInstruction : public AbstractJInstruction {
			public:
				JzInstruction(vector<string> &labels) : AbstractJInstruction(labels, "jz") { };
				uint16_t getBinary();
		};

		class JnInstruction : public AbstractJInstruction {
			public:
				JnInstruction(vector<string> &labels) : AbstractJInstruction(labels, "jn") { };
				uint16_t getBinary();
		};

		ostream &operator << (ostream &os, Instruction &i);
	}
}


#endif  // __INSTRUCTION_H__
