

#include <iostream>
#include <bitset>
#include <vector>
#include "Instruction.H"


using namespace v1processor;
using namespace std;


Instruction *InstructionFactory::newInstruction(vector<string> &labels, string id) {
	if (id == "loadi")
		return new LoadIInstruction(labels);
	else if (id == "load")
		return new LoadInstruction(labels);
	else if (id == "op")
		return new OpInstruction(labels);
	else if (id == "store")
		return new StoreInstruction(labels);
	else if (id == "push")
		return new PushInstruction(labels);
	else if (id == "pop")
		return new PopInstruction(labels);
	else if (id == "j")
		return new JInstruction(labels);
	else if (id == "jz")
		return new JzInstruction(labels);
	else if (id == "jn")
		return new JnInstruction(labels);
	else
		return NULL;
}


uint16_t Instruction::getRegister(string &registerName) {
	if (registerName == "ra")
		return 0;
	else if (registerName == "rb")
		return 1;
	else if (registerName == "sp")
		return 2;
	else if (registerName == "pc")
		return 3;
	else
		return 0;
}


uint16_t LoadIInstruction::getBinary() {
	return 0x8000 | (this->sourceValue & 0x7FFF);
}


uint16_t OpInstruction::getBinary() {
	uint16_t rd = this->getRegister(this->destinationRegister);
	uint16_t rs = this->getRegister(this->sourceRegister);
	uint16_t op = 0;
	if (this->operation == "assign")
		op = 0;
	else if (this->operation == "add")
		op = 1;
	else if (this->operation == "sub")
		op = 2;
	else if (this->operation == "and")
		op = 3;
	else if (this->operation == "or")
		op = 4;
	else if (this->operation == "xor")
		op = 5;
	else if (this->operation == "not")
		op = 6;
	else if (this->operation == "inc")
		op = 7;
	else if (this->operation == "dec")
		op = 8;
	else if (this->operation == "slr")
		op = 9;
	else if (this->operation == "sar")
		op = 10;
	else if (this->operation == "sll")
		op = 11;
	else if (this->operation == "addz")
		op = 12;
	else if (this->operation == "addn")
		op = 13;
	return 0x2000 | (rd << 8) | (rs << 4) | op;
}


uint16_t JInstruction::getBinary() {
	return 0x6000 | (this->destinationValue & 0x0FFF);
}


uint16_t JzInstruction::getBinary() {
	return 0x7000 | (this->destinationValue & 0x0FFF);
}


uint16_t JnInstruction::getBinary() {
	return 0x0000 | (this->destinationValue & 0x0FFF);
}


ostream &v1processor::operator << (ostream &os, Instruction &i) {
	for (vector<string>::iterator it = i.labels.begin(); it != i.labels.end(); it++)
		os << *it << ":" << endl;
	os << bitset<16>(i.getBinary()) << "\t";
	os << "\t" << i.mnemonic;
	if (i.mnemonic == "loadi") {
		LoadIInstruction *li = (LoadIInstruction *) &i;
		os << "\t\t";
		if (li->sourceLabel != "")
			os << li->sourceLabel;
		else
			os << li->sourceValue;
	}
	if (i.mnemonic == "op") {
		OpInstruction *pi = (OpInstruction *) &i;
		os << "\t\t" << pi->destinationRegister << ", " << pi->sourceRegister << ", " << pi->operation;
	}
	else if ((i.mnemonic == "j") || (i.mnemonic == "jz") || (i.mnemonic == "jn")) {
		AbstractJInstruction *ji = (AbstractJInstruction *) &i;
		os << "\t\t";
		if (ji->destinationLabel != "")
			os << ji->destinationLabel;
		else
			os << ji->destinationValue;
	}
	os << endl;
	return os;
}

