

#ifndef  __LEXICALANALYZER_H__
#define  __LEXICALANALYZER_H__


#include <iostream>
#include <sstream>
#include <queue>
#include "Exception.H"


extern "C++" {
	namespace v1processor {
		using namespace std;

		class LexicalAnalyzerException : public Exception {
			public:
				int line;
				int column;
				LexicalAnalyzerException(string message, int line, int column) : Exception(message), line(line), column(column) { };
		};

		ostream &operator << (ostream &os, const LexicalAnalyzerException &e);

		class Token {
			public:
				static const int NUM_RESERVED_WORDS = 28;
				static const string RESERVED_WORDS[NUM_RESERVED_WORDS];
				static const int TYPE_ID     = 51;
				static const int TYPE_COMMA  = 52;
				static const int TYPE_COLON  = 53;
				static const int TYPE_NUMBER = 54;
				static const int TYPE_EOF    = 55;
				int type;
				string image;
				int number;
				int line;
				int column;
				Token(int type, const string image, int line, int column) : type(type), image(image), line(line), column(column) { };
				Token(int number, int line, int column) : type(TYPE_NUMBER), number(number), line(line), column(column) { };
				bool isReservedWord() { return (this->type < 51); }
				static Token *newIdInstance(string image, int line, int column);
				static Token *newNumberInstance(string image, int line, int column);
		};

		ostream &operator << (ostream &os, const Token &v);

		class LexicalAnalyzer {
			protected:
				istream *is;
				static const int STATUS_IDLE                   = 1;
				static const int STATUS_LINE_COMMENT           = 2;
				static const int STATUS_PRE_MULTILINE_COMMENT  = 3;
				static const int STATUS_ID                     = 4;
				static const int STATUS_NUMBER                 = 5;
				static const int STATUS_MULTILINE_COMMENT      = 6;
				static const int STATUS_POST_MULTILINE_COMMENT = 7;
				static const int STATUS_STOP                   = 8;
				int status;
				int line, column;
				ostringstream oss;
				queue<Token *> tokenQueue;
			public:
				LexicalAnalyzer(istream &is);
				Token *getNextToken();
				void consumeNextToken();
		};
	}
}


#endif // __LEXICALANALYZER_H__
