

#ifndef  __PARSER_H__
#define  __PARSER_H__


#include <iostream>
#include <vector>
#include "Exception.H"
#include "Instruction.H"
#include "LexicalAnalyzer.H"


extern "C++" {
	namespace v1processor {
		using namespace std;

		class ParserException : public Exception {
			public:
				Token *token;
				ParserException(string message, Token *token) : Exception(message), token(token) { };
		};

		ostream &operator << (ostream &os, const ParserException &e);

		class SemanticException : public Exception {
			public:
				SemanticException(string message) : Exception(message) { };
		};

		ostream &operator << (ostream &os, const SemanticException &e);

		class Parser {
			protected:
				LexicalAnalyzer lexicalAnalyzer;
				InstructionFactory instructionFactory;
				Instruction *currentInstruction;
				Token *token;
				void matchToken(int type, string expected);
				void parseInstruction(vector<string> &labels);
				void parseOpcode(vector<string> &labels);
				void parseLoadIParameter();
				void parseOpParameters();
				void parseJParameter();
				void translateLabels();
			public:
				vector<Instruction *> instructions;
				Parser(istream &is);
				void parse();
		};
	}
}


#endif  // __PARSER_H__
