

#include <stdlib.h>
#include <iostream>
#include <map>
#include "Parser.H"


#undef  DEBUG


using namespace v1processor;
using namespace std;


ostream &v1processor::operator << (ostream &os, const ParserException &e) {
	os << "Parser exception: " << e.message << " token: " << *e.token << " at line " << e.token->line << ", column " << e.token->column;
	return os;
}


ostream &v1processor::operator << (ostream &os, const SemanticException &e) {
	os << "Semantic exception: " << e.message;
	return os;
}


Parser::Parser(istream &is) : lexicalAnalyzer(is) {
}


void Parser::matchToken(int type, string expected) {
	if (this->token == NULL) {
		ostringstream oss;
		oss << expected << " expected";
		throw ParserException(oss.str(), NULL);
	}
	if (this->token->type != type) {
		ostringstream oss;
		oss << expected << " expected";
		throw ParserException(oss.str(), this->token);
	}
	this->lexicalAnalyzer.consumeNextToken();
	this->token = this->lexicalAnalyzer.getNextToken();
}


void Parser::parseInstruction(vector<string> &labels) {
#ifdef  DEBUG
	cout << "parseInstruction" << endl;
#endif
	if (this->token->isReservedWord())
		this->parseOpcode(labels);
	else {
		string label = this->token->image;
		labels.push_back(label);
		this->lexicalAnalyzer.consumeNextToken();
		this->token = this->lexicalAnalyzer.getNextToken();
		this->matchToken(Token::TYPE_COLON, "':'");
		this->parseInstruction(labels);
	}
}


void Parser::parseOpcode(vector<string> &labels) {
#ifdef  DEBUG
	cout << "parseOpcode" << endl;
#endif
	if (this->token->isReservedWord()) {
		this->currentInstruction = this->instructionFactory.newInstruction(labels, this->token->image);
		if (this->currentInstruction == NULL)
			throw ParserException("invalid opcode", this->token);
		this->instructions.push_back(this->currentInstruction);
		this->lexicalAnalyzer.consumeNextToken();
		this->token = this->lexicalAnalyzer.getNextToken();
		if (this->currentInstruction->mnemonic == "loadi")
			this->parseLoadIParameter();
		if (this->currentInstruction->mnemonic == "op")
			this->parseOpParameters();
		else if (this->currentInstruction->mnemonic == "j")
			this->parseJParameter();
		else if (this->currentInstruction->mnemonic == "jz")
			this->parseJParameter();
		else if (this->currentInstruction->mnemonic == "jn")
			this->parseJParameter();
	}
	else
		throw ParserException("opcode expected", this->token);
}


void Parser::parseLoadIParameter() {
#ifdef  DEBUG
	cout << "parseLoadIParameter" << endl;
#endif
	if (this->token->type == Token::TYPE_ID) {
		((LoadIInstruction *) this->currentInstruction)->sourceLabel = this->token->image;
		this->lexicalAnalyzer.consumeNextToken();
		this->token = this->lexicalAnalyzer.getNextToken();
	}
	else if (this->token->type == Token::TYPE_NUMBER) {
		((LoadIInstruction *) this->currentInstruction)->sourceValue = this->token->number;
		this->lexicalAnalyzer.consumeNextToken();
		this->token = this->lexicalAnalyzer.getNextToken();
	}
	else
		throw ParserException("label or value expected", this->token);
}


void Parser::parseOpParameters() {
#ifdef  DEBUG
	cout << "parseOpParameters" << endl;
#endif
	if (this->token->isReservedWord()) {
		if ((this->token->image == "ra") || (this->token->image == "rb") || (this->token->image == "rc") || (this->token->image == "sp") || (this->token->image == "pc")) {
			((OpInstruction *) this->currentInstruction)->destinationRegister = this->token->image;
			this->lexicalAnalyzer.consumeNextToken();
			this->token = this->lexicalAnalyzer.getNextToken();
			this->matchToken(Token::TYPE_COMMA, "','");
			if (this->token->isReservedWord()) {
				if ((this->token->image == "ra") || (this->token->image == "rb") || (this->token->image == "rc") || (this->token->image == "sp") || (this->token->image == "pc")) {
					((OpInstruction *) this->currentInstruction)->sourceRegister = this->token->image;
					this->lexicalAnalyzer.consumeNextToken();
					this->token = this->lexicalAnalyzer.getNextToken();
					this->matchToken(Token::TYPE_COMMA, "','");
					if (this->token->isReservedWord()) {
						if ((this->token->image == "assign") || (this->token->image == "add") || (this->token->image == "sub") || (this->token->image == "and") || (this->token->image == "or") || (this->token->image == "xor") || (this->token->image == "not") || (this->token->image == "inc") || (this->token->image == "dec") || (this->token->image == "slr") || (this->token->image == "sar") || (this->token->image == "sll")) {
							((OpInstruction *) this->currentInstruction)->operation = this->token->image;
							this->lexicalAnalyzer.consumeNextToken();
							this->token = this->lexicalAnalyzer.getNextToken();
						}
						else
							throw ParserException("operation name expected (assign, add, sub, etc.)", this->token);
					}
					else
						throw ParserException("operation name expected (assign, add, sub, etc.)", this->token);
				}
				else
					throw ParserException("register name expected", this->token);
			}
			else
				throw ParserException("register name expected", this->token);
		}
		else
			throw ParserException("register name expected", this->token);
	}
	else
		throw ParserException("register name expected", this->token);
}


void Parser::parseJParameter() {
#ifdef  DEBUG
	cout << "parseJParameter" << endl;
#endif
	if (this->token->type == Token::TYPE_ID) {
		((JInstruction *) this->currentInstruction)->destinationLabel = this->token->image;
		this->lexicalAnalyzer.consumeNextToken();
		this->token = this->lexicalAnalyzer.getNextToken();
	}
	else if (this->token->type == Token::TYPE_NUMBER) {
		((JInstruction *) this->currentInstruction)->destinationValue = this->token->number;
		this->lexicalAnalyzer.consumeNextToken();
		this->token = this->lexicalAnalyzer.getNextToken();
	}
	else
		throw ParserException("destination label o value expected", this->token);
}


void Parser::parse() {
#ifdef  DEBUG
	cout << "parse" << endl;
#endif
	this->instructions.clear();
	this->token = this->lexicalAnalyzer.getNextToken();
	while (this->token->type != Token::TYPE_EOF) {
		vector<string> labels;
		this->parseInstruction(labels);
	}
	this->translateLabels();
}


void Parser::translateLabels() {
	map<string, int> labels;
	// fill labels map
	int address = 0;
	for (vector<Instruction *>::iterator it = this->instructions.begin(); it < this->instructions.end(); it++, address++) {
		Instruction *i = *it;
		if (i->labels.size() > 0) {
			for (vector<string>::iterator it2 = i->labels.begin(); it2 < i->labels.end(); it2++)
				labels[*it2] = address;
		}
	}
	// replace instruction's label with values
	address = 0;
	for (vector<Instruction *>::iterator it = this->instructions.begin(); it < this->instructions.end(); it++, address++) {
		Instruction *i = *it;
		if (i->mnemonic == "loadi") {
			LoadIInstruction *li = (LoadIInstruction *) i;
			if (li->sourceLabel != "") {
				if (labels.count(li->sourceLabel) > 0) {
					li->sourceValue = labels[li->sourceLabel];
					li->sourceLabel = "";
				}
				else {
					ostringstream oss;
					oss << "unable to find label '" << li->sourceLabel << "'";
					throw SemanticException(oss.str());
				}
			}
		}
		else if ((i->mnemonic == "j") || (i->mnemonic == "jz") || (i->mnemonic == "jn")) {
			AbstractJInstruction *ji = (AbstractJInstruction *) i;
			if (ji->destinationLabel != "") {
				if (labels.count(ji->destinationLabel) > 0) {
					ji->destinationValue = labels[ji->destinationLabel] - address - 1;
					ji->destinationLabel = "";
				}
				else {
					ostringstream oss;
					oss << "unable to find label '" << ji->destinationLabel << "'";
					throw SemanticException(oss.str());
				}
			}
		}
	}
}
