

#include <stdint.h>
#include <iostream>
#include <iomanip>
#include <bitset>
#include <sstream>
#include "CPU.H"
#include "RuntimeException.H"


#undef  DEBUG


using namespace v1processor;
using namespace std;


CPU::CPU(MemoryUnit &memoryUnit) {
	this->memoryUnit = &memoryUnit;
}


void CPU::reset() {
	this->pc = 0;
	this->justReseted = true;
}


int16_t *CPU::getRegister(uint8_t r) {
	if (r == 0)
		return &this->ra;
	else if (r == 1)
		return &this->rb;
	else if (r == 2)
		return &this->rc;
	else if (r == 3)
		return &this->sp;
	else
		return &this->pc;
}


void CPU::step() {
	this->justReseted = false;
	this->ir = this->memoryUnit->getMem(this->pc);
	int16_t irPc = this->pc;
#ifdef  DEBUG
	cout << "PC = " << this->pc << ", IR = " << bitset<16>(this->ir) << endl;
#endif
	this->pc++;
	int16_t opCode = (this->ir >> 12) & 15;
#ifdef  DEBUG
	cout << "opCode = " << opCode << endl;
#endif
	if ((opCode & 0x08) == 0x08) {
		// loadi
		int16_t v = (this->ir & 0x7FFF);
		if (v & 0x4000)
			v |= 0x8000;   // se extiende el signo si es un valor negativo
#ifdef  DEBUG
		cout << "loadi " << v << endl;
#endif
		this->ra = v;
	}
	else if (opCode == 1) {
		// load
#ifdef  DEBUG
		cout << "load" << endl;
#endif
		this->ra = this->memoryUnit->getMem(this->rb);
	}
	else if (opCode == 2) {
		// op
		uint8_t rd = (this->ir >> 8) & 7;
		int16_t *dst = this->getRegister(rd);
		uint8_t rs = (this->ir >> 4) & 7;
		int16_t *src = this->getRegister(rs);
		int16_t op = this->ir & 15;
		string opStr;
		if (op == 0) {
			*dst = *src;
			opStr = "assign";
		}
		else if (op == 1) {
			*dst = *dst + *src;
			opStr = "add";
		}
		else if (op == 2) {
			*dst = *dst - *src;
			opStr = "sub";
		}
		else if (op == 3) {
			*dst = *dst & *src;
			opStr = "and";
		}
		else if (op == 4) {
			*dst = *dst | *src;
			opStr = "or";
		}
		else if (op == 5) {
			*dst = *dst ^ *src;
			opStr = "xor";
		}
		else if (op == 6) {
			*dst = ~*src;
			opStr = "not";
		}
		else if (op == 7) {
			*dst = *src + 1;
			opStr = "inc";
		}
		else if (op == 8) {
			*dst = *src - 1;
			opStr = "dec";
		}
		else if ((op == 9) || (op == 10)) {
			*dst = *src >> 1;
			opStr = "slr/sar";
		}
		else if (op == 11) {
			*dst = *src << 1;
			opStr = "sll";
		}
		else if (op == 12) {
			if (this->ra == 0)
				*dst = *dst + *src;
			opStr = "addz";
		}
		else if (op == 13) {
			if (this->ra < 0)
				*dst = *dst + *src;
			opStr = "addn";
		}
#ifdef  DEBUG
		cout << "op " << ((int) rd) << ", " << ((int) rs) << ", " << opStr << endl;
#endif
	}
	else if (opCode == 3) {
		// store
#ifdef  DEBUG
		cout << "store" << endl;
#endif
		this->memoryUnit->setMem(this->rb, this->ra);
	}
	else if (opCode == 4) {
		// push
#ifdef  DEBUG
		cout << "push" << endl;
#endif
		this->sp++;
		this->memoryUnit->setMem(this->sp, this->ra);
	}
	else if (opCode == 5) {
		// pop
#ifdef  DEBUG
		cout << "pop" << endl;
#endif
		this->ra = this->memoryUnit->getMem(this->sp);
		this->sp--;
	}
	else if ((opCode == 6) || (opCode == 7) || (opCode == 0)) {
		// j, jz, jn
		int16_t v = (this->ir & 0x0FFF);
		if (v & 0x0800)
			v |= 0xF000;   // se extiende el signo si es un valor negativo
		if (opCode == 6) {
#ifdef  DEBUG
			cout << "j " << v << endl;
#endif
			this->pc += v;
		}
		else if (opCode == 7) {
#ifdef  DEBUG
			cout << "jz " << v << endl;
#endif
			if (this->ra == 0)
				this->pc += v;
		}
		else {
#ifdef  DEBUG
			cout << "jn " << v << endl;
#endif
			if (this->ra < 0)
				this->pc += v;
		}
	}
	else {
#ifdef  DEBUG
		cout << "!" << endl;
#endif
		ostringstream oss;
		oss << "unrecognized opcode " << opCode;
		throw RuntimeException(oss.str(), irPc);
	}
}


ostream &v1processor::operator << (ostream &os, CPU &cpu) {
	if (cpu.isJustReseted())
		os << "just after reset: ";
	else {
		os << "after '";
		int16_t opCode = (cpu.ir >> 12) & 15;
		if ((opCode & 0x08) == 0x08) {
			// loadi
			int16_t v = (cpu.ir & 0x7FFF);
			if (v & 0x4000)
				v |= 0x8000;   // se extiende el signo si es un valor negativo
			os << "loadi " << v;
		}
		else if (opCode == 1) {
			// load
			os << "load";
		}
		else if (opCode == 2) {
			// op
			int rd = (cpu.ir >> 8) & 7;
			int rs = (cpu.ir >> 4) & 7;
			int op = cpu.ir & 15;
			string opStr;
			if (op == 0)
				opStr = "assign";
			else if (op == 1)
				opStr = "add";
			else if (op == 2)
				opStr = "sub";
			else if (op == 3)
				opStr = "and";
			else if (op == 4)
				opStr = "or";
			else if (op == 5)
				opStr = "xor";
			else if (op == 6)
				opStr = "not";
			else if (op == 7)
				opStr = "inc";
			else if (op == 8)
				opStr = "dec";
			else if ((op == 9) || (op == 10))
				opStr = "slr/sar";
			else if (op == 11)
				opStr = "sll";
			else if (op == 12)
				opStr = "addz";
			else if (op == 13)
				opStr = "addn";
			os << "op " << ((int) rd) << ", " << ((int) rs) << ", " << opStr;
		}
		else if (opCode == 3) {
			// store
			os << "store";
		}
		else if (opCode == 4) {
			// push
			os << "push";
		}
		else if (opCode == 5) {
			// pop
			os << "pop";
		}
		else if ((opCode == 6) || (opCode == 7) || (opCode == 0)) {
			// j, jz, jn
			int16_t v = (cpu.ir & 0x0FFF);
			if (v & 0x0800)
				v |= 0xF000;   // se extiende el signo si es un valor negativo
			if (opCode == 6)
				os << "j " << v;
			else if (opCode == 7)
				os << "jz " << v;
			else
				os << "jn " << v;
		}
		else
			os << "unrecognized opcode " << opCode;
		os << "': ";
	}
	os << "RA=" << cpu.ra << ", RB=" << cpu.rb << ", SP=" << cpu.sp << ", PC=" << cpu.pc << ", IR=" << bitset<16>(cpu.ir);
	return os;
}
