

#ifndef  __MEMORYUNIT_H__
#define  __MEMORYUNIT_H__


#include <stdint.h>
#include <vector>
#include <string>


extern "C++" {
	namespace v1processor {
		using namespace std;

		class Port {
			public:
				bool isOutput;
				Port(bool isOutput) : isOutput(isOutput) { };
				virtual int16_t getAddress() = 0;
		};

		class IPort : public Port {
			public:
				IPort() : Port(false) { };
				virtual int16_t getValue() = 0;
		};

		class OPort : public Port {
			public:
				OPort() : Port(true) { };
				virtual void setValue(int16_t v) = 0;
		};

		class MemoryUnit {
			public:
				static const int16_t ROM_SIZE = 512;
				static const int16_t RAM_SIZE = 512;
				int16_t ra, rb, rc, pc;
				int16_t getMem(int16_t address);
				void setMem(int16_t address, int16_t data);
				void registerPort(Port &port);
				int16_t *getRom();
			protected:
				int16_t rom[ROM_SIZE];
				int16_t ram[RAM_SIZE];
				vector<Port *> ports;
				Port *portLookup(int16_t address);
				IPort *iPortLookup(int16_t address);
				OPort *oPortLookup(int16_t address);
		};
	}
}


#endif  // __MEMORYUNIT_H__
