

#include <stdlib.h>
#include <string>
#include <iostream>
#include <sstream>
#include <iomanip>
#include "MemoryUnit.H"
#include "RuntimeException.H"


using namespace std;
using namespace v1processor;


void MemoryUnit::registerPort(Port &port) {
	this->ports.push_back(&port);
}


int16_t *MemoryUnit::getRom() {
	return this->rom;
}


Port *MemoryUnit::portLookup(int16_t address) {
	for (vector<Port *>::iterator it = this->ports.begin(); it != this->ports.end(); it++) {
		Port *port = *it;
		if (port->getAddress() == address)
			return port;
	}
	return NULL;
}


IPort *MemoryUnit::iPortLookup(int16_t address) {
	Port *p = this->portLookup(address);
	if (p != NULL) {
		if (!p->isOutput)
			return (IPort *) p;
	}
	return NULL;
}


OPort *MemoryUnit::oPortLookup(int16_t address) {
	Port *p = this->portLookup(address);
	if (p != NULL) {
		if (p->isOutput)
			return (OPort *) p;
	}
	return NULL;
}


int16_t MemoryUnit::getMem(int16_t address) {
	IPort *iPort = this->iPortLookup(address);
	if (iPort != NULL)
		return iPort->getValue();
	else {
		if (address < ROM_SIZE)
			return this->rom[address];
		else
			return this->ram[address - this->ROM_SIZE];
	}
}


void MemoryUnit::setMem(int16_t address, int16_t data) {
	OPort *oPort = this->oPortLookup(address);
	if (oPort != NULL) {
		oPort->setValue(data);
	}
	else {
		if (address < ROM_SIZE) {
			ostringstream oss;
			oss << "unable to write at ROM address = 0x" << hex << setw(4) << setfill('0') << ((int) address) << dec << " (" << ((int) address) << " dec)";
			throw RuntimeException(oss.str(), this->pc);
		}
		else
			this->ram[address - this->ROM_SIZE] = data;
	}
}
