

#include <stdint.h>
#include <stdlib.h>
#include <iostream>
#include <iomanip>
#include <fstream>
#include "CPU.H"
#include "MemoryUnit.H"
#include "RuntimeException.H"


using namespace v1processor;
using namespace std;


class Led : public OPort {
	public:
		int16_t getAddress();
		void setValue(int16_t v);
};


int16_t Led::getAddress() {
	return 600;
}


void Led::setValue(int16_t v) {
	cout << "led = " << ((int) v) << endl;
}


int main(int argc, char *argv[]) {
	if (argc < 2) {
		cerr << "Syntax: " << argv[0] << " romfile [-s]" << endl;
		return 1;
	}
	bool stepMode = false;
	string trash;
	if (argc > 2) {
		if (string(argv[2]) == "-s")
			stepMode = true;
	}
	MemoryUnit memoryUnit;
	ifstream ifs(argv[1], ifstream::binary);
	ifs.read((char *) memoryUnit.getRom(), MemoryUnit::ROM_SIZE);
	ifs.close();
	Led led;
	memoryUnit.registerPort(led);
	CPU cpu(memoryUnit);
	cpu.reset();
	if (stepMode) {
		cout << cpu << endl;
		cin.get();
	}
	while (true) {
		try {
			cpu.step();
			if (stepMode) {
				cout << cpu << endl;
				cin.get();
			}
		}
		catch (RuntimeException &re) {
			cerr << "Exception at " << re.pc << ": " << re.message << endl;
			return 1;
		}
	}
	return 0;
}
